/******************************************************************************
 * Icinga 2                                                                   *
 * Copyright (C) 2018 Herrmann & Lenz Solutions GmbH (https://hl-solutions.de)*
 *                                                                            *
 * This program is free software; you can redistribute it and/or              *
 * modify it under the terms of the GNU General Public License                *
 * as published by the Free Software Foundation; either version 2             *
 * of the License, or (at your option) any later version.                     *
 *                                                                            *
 * This program is distributed in the hope that it will be useful,            *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 * GNU General Public License for more details.                               *
 *                                                                            *
 * You should have received a copy of the GNU General Public License          *
 * along with this program; if not, write to the Free Software Foundation     *
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ******************************************************************************/

#ifndef HLMON_H
#define HLMON_H

#include <SimpleAmqpClient/SimpleAmqpClient.h>

#include "hlmon/hlmonsender-ti.hpp"
#include "icinga/service.hpp"
#include "base/timer.hpp"
#include <fstream>

namespace icinga
{

/**
 * @ingroup hlmon
 */
class HLMonSender : public ObjectImpl<HLMonSender>
{
public:
	DECLARE_OBJECT(HLMonSender);
	DECLARE_OBJECTNAME(HLMonSender);

	virtual void Start(bool runtimeCreated);

	static Value HlmonMessageHandler(const MessageOrigin::Ptr& origin, const Dictionary::Ptr& params);
	static void StatsFunc(const Dictionary::Ptr& status, const Array::Ptr& perfdata);

private:
	void CheckResultHandler(const Checkable::Ptr& service, const CheckResult::Ptr& cr);
	void SetAcknowledgementHandler(const Checkable::Ptr& checkable);
	void ClearAcknowledgementHandler(const Checkable::Ptr& checkable);

	void AddCommentHandler(const Comment::Ptr& comment);
	void RemoveCommentHandler(const Comment::Ptr& comment);

	void AddDowntimeHandler(const Downtime::Ptr& downtime);
	void TriggerDowntimeHandler(const Downtime::Ptr& downtime);
	void RemoveDowntimeHandler(const Downtime::Ptr& downtime);

	void ChangeConfigHandler(void);

	void GenerateAndSendEvent(const Checkable::Ptr& checkable, const CheckResult::Ptr& cr);
	void GenerateAndSendStat(const Checkable::Ptr& checkable);
	void GenerateAndSendSchedulerStat(void);
	void SendToRabbitMQ(const String& line);

	AmqpClient::Channel::ptr_t getChannel();
	AmqpClient::Channel::ptr_t channel;
};

}

#endif /* HLMON_H */
